package tofly.toflylinks.mqtt_transfer;

import com.jayway.jsonpath.JsonPath;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.core.task.TaskExecutor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

@Slf4j
@Component
@AllArgsConstructor
public class MqttTransferService {

    private static int connectionTimeout = 10; // 连接超时时间（秒）
    private static int keepAliveInterval = 20; // 会话心跳时间(秒)
    private static MqttClient fromClient;

    private final TransferProperties transferProperties;
    private final ThreadPoolTaskExecutor taskExecutor;
    public static Monitor monitor = new Monitor();

    public void transfer() throws MqttException, InterruptedException {
        try {
            TransferProperties.Server fromServer = transferProperties.getFromServer();
            TransferProperties.Server toServer = transferProperties.getToServer();
            fromClient = connect(fromServer, new TransferFromMqttMessageCallback(fromServer, toServer, taskExecutor, monitor));
        } catch (MqttException e) {
            log.info("连接异常，原因:{}，稍后重试...", e.getMessage());
            Thread.sleep(5000);
            transfer();
        }
    }

    @Scheduled(cron = "${monitor.cron_expression:0/30 * * * * ?}")
    public void monitorTask() {
        log.info("{}", monitor);
    }

    private static MqttClient connect(TransferProperties.Server server, MqttCallback mqttCallback) throws MqttException {
        String uri = server.getUri();
        String clientId = server.getClientId();
        String username = server.getUsername();
        String password = server.getPassword();
        MqttClient client = new MqttClient(uri, clientId, new MemoryPersistence());
        MqttConnectOptions options = new MqttConnectOptions();
        options.setCleanSession(true);
        options.setAutomaticReconnect(true);
        options.setUserName(username);
        options.setPassword(password.toCharArray());
        options.setConnectionTimeout(connectionTimeout); // 设置超时时间
        options.setKeepAliveInterval(keepAliveInterval); // 设置会话心跳时间
        if (mqttCallback != null) {
            client.setCallback(mqttCallback);
        }
        client.connect(options);
        return client;
    }

    private static class TransferFromMqttMessageCallback implements MqttCallbackExtended {
        private final TransferProperties.Server fromServer;
        private final TransferProperties.Server toServer;
        private final TaskExecutor taskExecutor;
        private final Monitor monitor;

        public TransferFromMqttMessageCallback(TransferProperties.Server fromServer, TransferProperties.Server toServer, ThreadPoolTaskExecutor taskExecutor, Monitor monitor) {
            this.toServer = toServer;
            this.fromServer = fromServer;
            this.taskExecutor = taskExecutor;
            this.monitor = monitor;
        }

        public void connectionLost(Throwable cause) {
            log.warn("连接丢失:", cause);
        }

        public void deliveryComplete(IMqttDeliveryToken token) {
        }

        public void messageArrived(String topic, MqttMessage message) {
            byte[] payload = message.getPayload();
            String json = new String(payload);
            monitor.setLastTime(LocalDateTime.now());
            monitor.getReceive().getAndIncrement();
            monitor.setLastData(json);
            log.trace(json);
            String jsonPathExpression = "$.tofly_device_id";
            Object toflyDeviceId = JsonPath.read(json, jsonPathExpression);
            taskExecutor.execute(() -> {
                TransferProperties.Server server = null;
                MqttClient toClient = null;
                try {
                    server = toServer.clone();
                    server.setClientId(String.valueOf(toflyDeviceId));
                    toClient = connect(server, null);
                    MqttMessage mm = new MqttMessage(payload);
                    mm.setQos(2);
                    toClient.publish(server.getTopic(), mm);
                    monitor.getSendSuccess().getAndIncrement();
                } catch (Exception e) {
                    monitor.getSendFault().getAndIncrement();
                    log.error(String.format("数据中转异常:%s,%s", server, json), e);
                } finally {
                    if (toClient != null) {
                        try {
                            toClient.disconnect(1000);
                            toClient.close(true);
                        } catch (MqttException e) {
                            log.warn("关闭客户端异常", e);
                        }
                    }
                }
            });
        }

        @Override
        public void connectComplete(boolean reconnect, String serverURI) {
            log.info("MQTT服务连接成功:{}", serverURI);
            String topic = fromServer.getTopic();
            try {
                fromClient.subscribe(topic, 2);
                log.info("MQTT主题({})订阅成功！！！", topic);
            } catch (Exception e) {
                log.error(String.format("MQTT主题(%s)订阅失败！！！", topic), e);
            }
        }
    }
}
